from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata


class PhoneNumberGeneratorMetadata(BaseMetadata):
    def __init__(self, use_us_phone_number_generator: Optional[bool] = False, replace_invalid_numbers: Optional[bool] = True):
        super().__init__()
        self.use_us_phone_number_generator = use_us_phone_number_generator
        self.replace_invalid_numbers = replace_invalid_numbers
    
    def to_payload(self, default: "PhoneNumberGeneratorMetadata") -> Dict:
        result = super().to_payload(default)

        result["useUsPhoneNumberGenerator"] = self.use_us_phone_number_generator
        result["replaceInvalidNumbers"] = self.replace_invalid_numbers

        return result
    

default_phone_number_generator_metadata = PhoneNumberGeneratorMetadata()
def phone_number_generator_metadata_to_payload(metadata: PhoneNumberGeneratorMetadata) -> Dict:
    return metadata.to_payload(default_phone_number_generator_metadata)


from typing import List, Dict, Optional

from tonic_textual.classes.generator_metadata.base_date_time_generator_metadata import BaseDateTimeGeneratorMetadata
from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata
from tonic_textual.classes.generator_metadata.timestamp_shift_metadata import TimestampShiftMetadata, default_timestamp_shift_metadata


class DateTimeGeneratorMetadata(BaseDateTimeGeneratorMetadata):
    def __init__(self, additional_date_formats: Optional[List[str]] = [], apply_constant_shift_to_document: Optional[bool] = False, timestamp_shift_metadata: Optional[TimestampShiftMetadata] = default_timestamp_shift_metadata):
        super().__init__()
        self._date_time_transformation = "TimestampShift"
        self._metadata = timestamp_shift_metadata
        self.additional_date_formats = additional_date_formats
        self.apply_constant_shift_to_document = apply_constant_shift_to_document

    def to_payload(self, default: "DateTimeGeneratorMetadata") -> Dict:
        result = super().to_payload(default)
        
        result["dateTimeTransformation"] = self._date_time_transformation
        result["metadata"] = self._metadata.to_payload(default._metadata)
        result["additionalDateFormats"] = self.additional_date_formats    
        result["applyConstantShiftToDocument"] = self.apply_constant_shift_to_document

        return result

    def date_time_transformation(self) -> str:
        return self._date_time_transformation

    def metadata(self) -> BaseMetadata:
        return self._metadata

default_date_time_metadata = DateTimeGeneratorMetadata()
def date_time_metadata_to_payload(metadata: DateTimeGeneratorMetadata) -> Dict:
    return metadata.to_payload(default_date_time_metadata)



from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata


class AgeShiftMetadata(BaseMetadata):
    def __init__(self, age_shift_in_years: Optional[int] = 7):
        super().__init__()
        self.age_shift_in_years = age_shift_in_years

    def to_payload(self, default: "AgeShiftMetadata") -> Dict:
        result = super().to_payload(default)

        result["ageShiftInYears"] = self.age_shift_in_years        

        return result


default_age_shift_metadata = AgeShiftMetadata()
def age_shift_metadata_to_payload(metadata: AgeShiftMetadata) -> Dict:
    return metadata.to_payload(default_age_shift_metadata)


from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata

class TimestampShiftMetadata(BaseMetadata):

    def __init__(self, timestamp_shift_in_days: Optional[int] = 7):
        super().__init__()
        self.timestamp_shift_in_days = timestamp_shift_in_days

    def to_payload(self, default: "TimestampShiftMetadata") -> Dict:
        result = super().to_payload(default)
        
        result["timestampShiftInDays"] = self.timestamp_shift_in_days        
        return result

default_timestamp_shift_metadata = TimestampShiftMetadata()
def timestamp_shift_metadata_to_payload(metadata: TimestampShiftMetadata) -> Dict:
    return metadata.to_payload(default_timestamp_shift_metadata)


from typing import List
from tonic_textual.classes.dataset import Dataset
from urllib.parse import urlencode
import requests


class DatasetService:
    def __init__(self, client):
        self.client = client

    def get_dataset(self, dataset_name):
        with requests.Session() as session:
            params = {"datasetName": dataset_name}
            dataset = self.client.http_get(
                "/api/dataset/get_dataset_by_name?" + urlencode(params), session=session
            )
            return Dataset(
                self.client,
                dataset["id"],
                dataset["name"],
                dataset["files"],
                dataset["generatorSetup"],
                dataset["generatorMetadata"],
                dataset["labelBlockLists"],
                dataset["labelAllowLists"],
                dataset["docXImagePolicy"],
                dataset["docXCommentPolicy"],
                dataset["docXTablePolicy"],
                dataset["pdfSignaturePolicy"],
                dataset["pdfSynthModePolicy"],
            )

    def get_all_datasets(self) -> List[Dataset]:
        with requests.Session() as session:
            datasets = self.client.http_get("/api/dataset", session=session)

            return [
                Dataset(
                    self.client,
                    dataset["id"],
                    dataset["name"],
                    dataset["files"],
                    dataset["generatorSetup"],
                    dataset["generatorMetadata"],
                    dataset["labelBlockLists"],
                    dataset["labelAllowLists"],
                    dataset["docXImagePolicy"],
                    dataset["docXCommentPolicy"],
                    dataset["docXTablePolicy"],
                    dataset["pdfSignaturePolicy"],
                    dataset["pdfSynthModePolicy"],
                )
                for dataset in datasets
            ]



from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata


class HipaaAddressGeneratorMetadata(BaseMetadata):
    def __init__(self, use_non_hipaa_address_generator: Optional[bool] = False, replace_truncated_zeros_in_zip_code: Optional[bool] = True, realistic_synthetic_values: Optional[bool] = True):
        super().__init__()
        self.use_non_hipaa_address_generator = use_non_hipaa_address_generator
        self.replace_truncated_zeros_in_zip_code = replace_truncated_zeros_in_zip_code
        self.realistic_synthetic_values = realistic_synthetic_values
    
    def to_payload(self, default: "HipaaAddressGeneratorMetadata") -> Dict:
        result = super().to_payload(default)

        result["useNonHipaaAddressGenerator"] = self.use_non_hipaa_address_generator
        result["replaceTruncatedZerosInZipCode"] = self.replace_truncated_zeros_in_zip_code        
        result["realisticSyntheticValues"] = self.realistic_synthetic_values

        return result
    
default_hipaa_address_metadata = HipaaAddressGeneratorMetadata()
def hipaa_address_metadata_to_payload(metadata: HipaaAddressGeneratorMetadata) -> Dict:
    return metadata.to_payload(default_hipaa_address_metadata)


from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.age_shift_metadata import AgeShiftMetadata, default_age_shift_metadata
from tonic_textual.classes.generator_metadata.base_date_time_generator_metadata import BaseDateTimeGeneratorMetadata
from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata


class PersonAgeGeneratorMetadata(BaseDateTimeGeneratorMetadata):
    def __init__(self, age_shift_metadata: Optional[AgeShiftMetadata] = default_age_shift_metadata):
        super().__init__()
        self._date_time_transformation = "AgeShift"
        self._metadata = age_shift_metadata

    def to_payload(self, default: "PersonAgeGeneratorMetadata") -> Dict:
        result = super().to_payload(default)

        result["dateTimeTransformation"] = self._date_time_transformation
        result["metadata"] = self._metadata.to_payload(default._metadata)

        return result


    def date_time_transformation(self) -> str:
        return "AgeShift"

    def metadata(self) -> BaseMetadata:
        return self._metadata

default_person_age_metadata = PersonAgeGeneratorMetadata()
def person_age_metadata_to_payload(metadata: PersonAgeGeneratorMetadata) -> Dict:
    return metadata.to_payload(default_person_age_metadata)


from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata


class NumericValueGeneratorMetadata(BaseMetadata):
    def __init__(self, use_oracle_integer_pk_generator: Optional[bool] = False):
        super().__init__()
        self.use_oracle_integer_pk_generator = use_oracle_integer_pk_generator

    def to_payload(self, default: "NumericValueGeneratorMetadata") -> Dict:
        result = super().to_payload(default)

        result["useOracleIntegerPkGenerator"] = self.use_oracle_integer_pk_generator

        return result


default_numeric_value_generator_metadata = NumericValueGeneratorMetadata()
def numeric_value_generator_metadata_to_payload(metadata: NumericValueGeneratorMetadata) -> Dict:
    return metadata.to_payload(default_numeric_value_generator_metadata)


from enum import Enum


class docx_image_policy(Enum):
    redact = 1
    ignore = 2
    remove = 3


class docx_comment_policy(Enum):
    remove = 1
    ignore = 2


class docx_table_policy(Enum):
    redact = 1
    remove = 2


class pdf_signature_policy(Enum):
    redact = 1
    ignore = 2

class pdf_synth_mode_policy(Enum):
    V1 = 1
    V2 = 2



from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata


class BaseDateTimeGeneratorMetadata(BaseMetadata):
    def __init__(self, scramble_unrecognized_dates: Optional[bool] = True):
        super().__init__()
        self.scramble_unrecognized_dates = scramble_unrecognized_dates

    def to_payload(self, default: "BaseDateTimeGeneratorMetadata") -> Dict:
        result = super().to_payload(default)

        if self.scramble_unrecognized_dates != default.scramble_unrecognized_dates:
            result["scrambleUnrecognizedDates"] = self.scramble_unrecognized_dates

        return result

    def date_time_transformation(self) -> str:
        raise NotImplementedError

    def metadata(self) -> BaseMetadata:
        raise NotImplementedError



from typing import Dict

from tonic_textual.enums.casing import Casing
from tonic_textual.enums.generator_version import GeneratorVersion


class BaseMetadata:
    def __init__(self):
        self.generator_version = GeneratorVersion.V1
        self.custom_generator = None
        self.styles = []
        self.casing = Casing.MixedCase

    def to_payload(self, default: "BaseMetadata") -> Dict:
        result = dict()

        if self.generator_version != default.generator_version:
            result["generatorVersion"] = self.generator_version

        if self.custom_generator != default.custom_generator:
            result["customGenerator"] = self.custom_generator

        if self.styles != default.styles:
            result["styles"] = self.styles

        if self.casing != default.casing:
            result["casing"] = self.casing

        return result

default_base_metadata = BaseMetadata()
def base_metadata_to_payload(metadata: BaseMetadata) -> Dict:
    return metadata.to_payload(default_base_metadata)


from typing import Dict, Optional

from tonic_textual.classes.generator_metadata.base_metadata import BaseMetadata


class NameGeneratorMetadata(BaseMetadata):
    def __init__(self, is_consistency_case_sensitive: Optional[bool] = False, preserve_gender: Optional[bool] = False):
        super().__init__()
        self.is_consistency_case_sensitive = is_consistency_case_sensitive if not None else False
        self.preserve_gender = preserve_gender if not None else True

    def to_payload(self, default: "NameGeneratorMetadata") -> Dict:
        result = super().to_payload(default)

        result["isConsistencyCaseSensitive"] = self.is_consistency_case_sensitive
        result["preserveGender"] = self.preserve_gender

        return result

default_name_generator_metadata = NameGeneratorMetadata()
def name_generator_metadata_to_payload(metadata: NameGeneratorMetadata) -> Dict:
    return metadata.to_payload(default_name_generator_metadata)